import json
import html
from .feature import parse


class BedTable:

    # Always remember the *self* argument
    def __init__(self, bed_file):

        self.features = []

        featureList = parse(bed_file)
        unique_id = 0
        for var in featureList:
            self.features.append((var, unique_id))
            unique_id += 1

    def to_JSON(self):

        headers = ["unique_id", "Start", "End", "Name"]
        rows = []

        for tuple in self.features:
            feature = tuple[0]
            unique_id = tuple[1]
            rows.append([unique_id, feature.chr, feature.start+1, feature.end, html.escape(feature.name)])

        return json.dumps({
            "headers": headers,
            "rows": rows
        })



class JunctionBedTable:

    # Always remember the *self* argument
    def __init__(self, bed_file, info_columns = None):

        self.features = []
        self.table_columns =  info_columns or None
        featureList = parse(bed_file)
        unique_id = 1
        session_id = 1
        session_dict = {}
        for f in featureList:
            #expand name field
            name_tokens = f.name.split(";")
            for token in name_tokens:
                kv = token.split("=")
                key = kv[0]
                value = kv[1]
                setattr(f, key, value)

            #create new session ID?
            if hasattr(f, 'viewport'):
                self.features.append((f, unique_id))
                unique_id += 1
                viewport = f.viewport
                if viewport in session_dict:
                    sid = session_dict[viewport]
                else:
                    sid = str(session_id)
                    session_dict[viewport] = sid
                    session_id = session_id + 1
                f.session_id = sid

    def to_JSON(self):

        json_array = [];

        for tuple in self.features:

            feature = tuple[0]
            name_tokens = feature.name.split(";")
            if hasattr(feature, 'session_id'):
                unique_id = tuple[1]
                obj = {
                    "unique_id": unique_id,
                    "session_id": feature.session_id,
                    "viewport": feature.viewport,
                    "feature_locus": feature.chr + ":" + str(feature.start) + "-" + str(feature.end),
                    "Chrom": feature.chr,
                    "Start": feature.start + 1,
                    "End": feature.end
                }

                if self.table_columns == None:
                    for token in name_tokens:
                        kv = token.split("=")
                        key = kv[0]
                        if key != 'viewport':
                            value = kv[1]
                            obj[key] = value
                else:
                    dict = {}
                    for token in name_tokens:
                        kv = token.split("=")
                        dict[kv[0]] = kv[1]
                    for key in self.table_columns:
                        if key in dict:
                            obj[key] = dict[key]
                        else:
                            obj[key] = ''


                json_array.append(obj)

        normalized = normalize_json(self.table_columns, json_array)
        return json.dumps(normalized)


def normalize_json(info_fields, json_array):

    headers = ['unique_id', 'CHROM', 'POSITION', 'REF', 'ALT', 'ID']
    if info_fields is not None:
        for h in info_fields:
            if h == 'ANN':
                headers = headers + [ 'GENE', 'EFFECTS', 'IMPACT', 'TRANSCRIPT', 'GENE_ID', 'PROTEIN ALTERATION', 'DNA ALTERATION']
            else:
                headers.append(h)

    rows = []
    for json in json_array:
        r = []
        for h in headers:
            if h in json:
                r.append(json[h])
            else:
                r.append("")
        rows.append(r)

    return {
        "headers": headers,
        "rows": rows
    }




import unittest

from igv_reports import vcf
import pathlib

class VcfTest(unittest.TestCase):

    def test_getvariants(self):

        path = str((pathlib.Path(__file__).parent / "data/minigenome/variants.vcf").resolve())
        data = vcf.get_data(path)
        self.assertIsNotNone(data)


    def test_getvariants_remote(self):

        path = "https://s3.amazonaws.com/igv.org.test/reports/variants/cancer.vcf.gz"
        region = {"chr": "chr17", "start": 7673767, "end": 43071077}
        data = vcf.get_data(path, region)
        self.assertIsNotNone(data)




import unittest
import pathlib
from igv_reports import ideogram

class IdeogramTest(unittest.TestCase):

    def test_extract_chr(self):

        data_path = str((pathlib.Path(__file__).parent / 'data/cytoBandIdeo.txt').resolve())
        extracted = ideogram.get_data(data_path, {"chr": "chr1"})
        self.assertTrue(extracted.startswith('chr1'))






import io
import gzip
import requests
from . import feature, bam, vcf



def getreader(path, filetype=None):

    if not filetype:
        filetype = feature.infer_format(path)
    else:
        filetype = filetype.lower()

    if filetype == "bam" or filetype == "cram":
        return bam.BamReader(path)

    elif filetype == "vcf":
        return vcf.VcfReader(path)

    else:
        return feature.FeatureReader(path)




import pysam

def get_data(path, region = None):

    vcf = VcfReader(path)

    return vcf.slice(region)


class VcfReader:

    def __init__(self, path):

        self.file = pysam.VariantFile(path)

    def slice(self, region = None):

        vcf = self.file
        header = vcf.header

        fileString = str(header)

        if region == None:
            records = vcf.fetch()
        else:
            records = vcf.fetch(region['chr'], region['start'], region['end'])

        for rec in records:

            fileString += (str(rec))

        return fileString




import os
import sys
import json
import math
import argparse
from urllib.request import urlopen
from igv_reports import fasta, ideogram, datauri, tracks, feature, bam, vcf, utils
from igv_reports.varianttable import VariantTable
from igv_reports.bedtable import BedTable
from igv_reports.bedtable import JunctionBedTable
from igv_reports.generictable import GenericTable
from igv_reports.regions import parse_region
from igv_reports.feature import MockReader

def create_report(args):

    trackreaders = []

    # Read the variant data
    variants_file = args.sites

    if variants_file.endswith(".vcf") or variants_file.endswith (".vcf.gz"):
        table = VariantTable(variants_file, args.info_columns, args.info_columns_prefixes, args.sample_columns)

    elif variants_file.endswith(".bed") or variants_file.endswith(".bed.gz"):
        if args.type is not None and args.type == "junction":
            table = JunctionBedTable(variants_file, args.info_columns)
        else:
            table = BedTable(variants_file)

    elif variants_file.endswith(".maf") or variants_file.endswith(".maf.gz") or (args.sequence is not None and args.begin is not None and args.end is not None):
        table = GenericTable(variants_file, args.info_columns, args.sequence, args.begin, args.end, args.zero_based)
        # A hack -- since these file formats are not supported by igv.js mock up a bed style track
        flist = []
        for tuple in table.features:
            flist.append(tuple[0])
        trackreaders.append({
            "track": "variants.bed",
            "reader": MockReader(flist)
        })


    table_json = table.to_JSON()

    session_dict = {}

    # Create file readers for tracks.  This is done outside the loucs loop so initialization happens once

    if args.tracks is not None:
        for track in args.tracks:
            reader = utils.getreader(track)
            trackreaders.append({
                "track": track,
                "reader": reader
            })

    trackconfigs = []
    if args.track_config is not None:
        for trackobj in args.track_config:
            with open(trackobj) as f:
                data = json.load(f)
                for config in data:
                    reader = utils.getreader(config["url"])
                    trackconfigs.append({
                        "config": config,
                        "reader": reader
                    })



    # loop through variants creating an igv.js session for each one
    flanking = 0
    if args.flanking is not None:
        flanking = float(args.flanking)

    for tuple in table.features:

        feature = tuple[0]
        unique_id = tuple[1]
        if hasattr(feature, "session_id"):
            session_id = feature.session_id
        else:
            session_id = str(unique_id)

        if session_id not in session_dict:

            # Define a genomic region around the variant
            if hasattr(feature, "viewport"):
                region = parse_region(feature.viewport)
                chr = region["chr"]
                start = region["start"]
                end = region["end"]
            else:
                chr = feature.chr
                start = int (math.floor(feature.start - flanking / 2))
                start = max(start,1) # bound start to 1
                end = int (math.ceil(feature.end + flanking / 2))
                region = {
                    "chr": chr,
                    "start": start,
                    "end": end
                }

            # Fasta
            data = fasta.get_data(args.fasta, region)
            fa = '>' + chr + ':' + str(start) + '-' + str(end) + '\n' + data
            fasta_uri = datauri.get_data_uri(fa)
            fastaJson = {
                "fastaURL": fasta_uri,
            }

            # Ideogram
            if(args.ideogram):
                ideo_string = ideogram.get_data(args.ideogram, region)
                ideo_uri = datauri.get_data_uri(ideo_string)
                fastaJson["cytobandURL"] = ideo_uri


            # Initial locus, +/- 20 bases
            if(hasattr(feature, "viewport")):
                initial_locus = feature.viewport
            else:
                position = int(math.floor((feature.start + feature.end) / 2)) + 1   # center of region in 1-based coordinates
                initial_locus = chr + ":" + str(position)
            session_json = {
                "locus": initial_locus,
                "reference": fastaJson,
                "tracks": []
            }


            track_objects = []
            for tr in trackreaders:
                track = tr["track"]
                reader = tr["reader"]
                trackobj = tracks.get_track_json_dict(track)
                data = reader.slice(region)
                trackobj["url"] = datauri.get_data_uri(data)
                track_objects.append(trackobj)

            # Loop through user supplied track configs
            for tc in trackconfigs:
                trackobj = tc["config"];
                default_trackobj = tracks.get_track_json_dict(trackobj["url"]);
                if "type"  not in trackobj:
                    trackobj["type"] = default_trackobj["type"]
                if "format" not in trackobj:
                    trackobj["format"] = default_trackobj["format"]
                if "name" not in trackobj:
                    trackobj["name"] = default_trackobj["url"]
                reader = tc["reader"]
                data = reader.slice(region)
                trackobj["url"] = datauri.get_data_uri(data)
                track_objects.append(trackobj)

            track_order = 1
            for trackobj in track_objects:
                if(trackobj["type"] == "alignment"):
                    trackobj["height"] = 500
                    is_snv = feature.end - feature.start == 1
                    if (trackobj["type"]) == "alignment" and (args.sort is not None or is_snv) and (args.sort != 'NONE'):
                        sort_option = 'BASE' if args.sort is None else args.sort.upper()
                        trackobj["sort"] = {
                            "option": sort_option,
                            "chr": chr,
                            "position": str(feature.start + 1),
                            "direction": "ASC"
                     }
                if "order" not in trackobj:
                    trackobj["order"] = track_order
                session_json["tracks"].append(trackobj)
                track_order += 1



    # Build the session data URI

            session_string = json.dumps(session_json)
            session_uri = datauri.get_data_uri(session_string)
            session_dict[session_id] = session_uri

    session_dict = json.dumps(session_dict)

    template_file = args.template
    if None == template_file:
        if 'junction' == args.type:
            template_file = os.path.dirname(sys.modules['igv_reports'].__file__) + '/templates/junction_template.html'
        else:
            template_file = os.path.dirname(sys.modules['igv_reports'].__file__) + '/templates/variant_template.html'

    output_file = args.output

    standalone = args.standalone
    with open(template_file, "r") as f:
        data = f.readlines()

        with open(output_file, "w") as o:

            for i, line in enumerate(data):

                if args.title is not None and line.startswith("<!--title-->"):
                    o.write("<h1>" + args.title + "</h1>")

                if standalone:
                    if line.strip().startswith("<script") and ".js\"" in line:
                        inline_script(line, o, "js")
                        continue
                    elif line.strip().startswith("<link") and line.strip().endswith("css\">"):
                        inline_script(line, o, "css")
                        continue
                j = line.find('"@TABLE_JSON@"')
                if j >= 0:
                    line = line.replace('"@TABLE_JSON@"', table_json)

                j = line.find('"@SESSION_DICTIONARY@"')
                if j >= 0:
                    line = line.replace('"@SESSION_DICTIONARY@"', session_dict)

                o.write(line)


def inline_script(line, o, source_type):
    #<script type="text/javascript" src="https://igv.org/web/test/dist/igv.min.js"></script>
    if source_type == "js":
        s = line.find('src="')
        offset = 5
        o.write('<script type="text/javascript">\n')
    elif source_type == "css":
        s = line.find('href="')
        offset = 6
        o.write('<style type="text/css">\n')
    else:
        raise KeyError("Inline script must be either js- or css-file")
    if s > 0:
        e = line.find('">', s)
        url = line[s+offset:e]
        response = urlopen(url)
        content = response.read().decode('utf-8')
        response.close()
        o.write(content)
        if source_type == "js":
            o.write('</script>\n')
        else:
            o.write('</style>\n')
    else:
        raise ValueError("No file path in {l} for inline script.".format(l=line))


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("sites", help="vcf file defining variants, required")
    parser.add_argument("fasta", help="reference fasta file, required")
    parser.add_argument("--type", help="Report type.  Possible values are mutation and junctions.  Default is mutation")
    parser.add_argument("--ideogram", help="ideogram file in UCSC cytoIdeo format")
    parser.add_argument("--tracks", nargs="+", help="list of track files")
    parser.add_argument("--track-config", nargs="+", help="track json file")
    parser.add_argument("--sort", help="initial sort option for alignment tracks.   Supported values include  BASE, STRAND, INSERT_SIZE, and MATE_CHR. Default value is BASE for single nucleotide variants, no sorting otherwise.  See the igv.js documentation for more information. ")
    parser.add_argument("--template", help="html template file", default=None)
    parser.add_argument("--output", help="output file name", default="igvjs_viewer.html")
    parser.add_argument("--info-columns", nargs="+", help="list of VCF info field names to include in variant table")
    parser.add_argument("--info-columns-prefixes", nargs="+", help="list of prefixes of VCF info field names to include in variant table")
    parser.add_argument("--sample-columns", nargs="+", help="list of VCF sample/format field names to include in variant table")
    parser.add_argument("--flanking", help="genomic region to include either side of variant", default=1000)
    parser.add_argument("--standalone", help='Print more data', action='store_true')
    parser.add_argument("--title", help="optional title string")
    parser.add_argument("--sequence", help="Column of sequence (chromosome) name.  For tab-delimited sites file.", default=None)
    parser.add_argument("--begin", help="Column of start position.  For tab-delimited sites file.", default=None)
    parser.add_argument("--end", help="column of end position. For tab-delimited sites file.", default=None)
    parser.add_argument("--zero_based", help="Specify that the position in the data file is 0-based (e.g. UCSC files) rather than 1-based.", default=None)
    args = parser.parse_args()
    create_report(args)

if __name__ == "__main__":

    main()



import unittest
import pathlib
from igv_reports import varianttable, bedtable, generictable

class TableTest(unittest.TestCase):

    def test_get_header(self):

        vcf_path = str((pathlib.Path(__file__).parent / "data/minigenome/variants.vcf").resolve())
        table = varianttable.VariantTable(vcf_path)
        self.assertEqual(len(table.variants), 28)

    def test_bedtable(self):

        bed_file = str((pathlib.Path(__file__).parent / "data/minigenome/variants.bed").resolve())
        table = bedtable.BedTable(bed_file)

        json = table.to_JSON()
        self.assertEqual(len(table.features), 4)
        self.assertTrue(json)

    def test_gfftable(self):

        gff = str((pathlib.Path(__file__).parent / "data/minigenome/annotations.gtf.gz").resolve())
        table = bedtable.BedTable(gff)
        json = table.to_JSON()
        self.assertTrue(len(table.features), 2106)
        self.assertTrue(json)


    def test_maftable(self):

        maf_file = str((pathlib.Path(__file__).parent / "data/maf/tcga_test.maf").resolve())
        table = generictable.GenericTable(maf_file)

        json = table.to_JSON()
        self.assertEqual(len(table.features), 17)
        self.assertTrue(json)

    def test_maflite(self):

        maf_file = str((pathlib.Path(__file__).parent / "data/maf/test.maflite.tsv").resolve())

        info_columns = "chr,start,end,ref_allele,alt_allele,tumor_barcode"
        sequence = 1
        start = 2
        end = 3

        table = generictable.GenericTable(maf_file, info_columns, sequence, start, end)

        json = table.to_JSON()
        self.assertEqual(29, len(table.features))
        self.assertTrue(json)







import json
import pysam
import html
from .feature import Feature


class VariantTable:

    # Always remember the *self* argument
    def __init__(self, vcfFile, info_columns = None, info_columns_prefixes = None, sample_columns = None):

        vcf = pysam.VariantFile(vcfFile)

        self.info_fields =  info_columns or []
        self.info_field_prefixes = info_columns_prefixes or []
        self.sample_fields = sample_columns or []
        self.variants = []
        self.features = []   #Bed-like features

        for unique_id, var in enumerate(vcf.fetch()):
            self.variants.append((var, unique_id))
            chr = var.chrom
            start = var.pos - 1
            end = start + 1       #TODO -- handle structure variants and deletions > 1 base
            self.features.append((Feature(chr, start, end, ''), unique_id))

    def to_JSON(self):


        json_array = [];

        for variant, unique_id in self.variants:

            escaped_alts = []
            for alt in variant.alts:
                escaped_alts.append(html.escape(alt))


            obj = {
                'unique_id': unique_id,
                'CHROM': variant.chrom,
                'POSITION': variant.pos,
                'REF': html.escape(variant.ref),
                'ALT': ','.join(escaped_alts),
                'ID': ''
            }

            if variant.id is not None:
                obj['ID'] = render_ids(variant.id)

            for h in self.info_fields:
                if h in variant.info:
                    if h == 'ANN':
                        genes, effects, impacts, transcript, gene_id, aa_alt, nt_alt = decode_ann(variant)
                        obj['GENE'] = genes
                        obj['EFFECTS'] = effects
                        obj['IMPACT'] = impacts
                        obj['TRANSCRIPT'] = transcript
                        obj['GENE_ID'] = gene_id
                        obj['PROTEIN ALTERATION'] = aa_alt
                        obj['DNA ALTERATION'] = nt_alt
                    elif h == 'COSMIC_ID':
                        cid = variant.info[h]
                        if cid is not None:
                            if isinstance(cid, str) :
                                return render_id(cid)
                            elif len(cid) == 1:
                                obj[h] = render_id(cid[0])
                            else:
                                tmp = ''
                                for c in cid:
                                    tmp = tmp + render_id(c) + ','
                                obj[h] = tmp
                    else:
                        obj[h] = render_values(variant.info[h])
                else:
                    obj[h] = ''

            for h in self.info_field_prefixes:
                for field in variant.info:
                    if field.startswith(h):
                        obj[field] = render_values(variant.info[field])

            for h in self.sample_fields:
                for sample, values in variant.samples.items():
                    v = ''
                    try:
                        v = values[h]
                    except KeyError:
                        # ignore if key is not present
                        pass

                    obj[f'{sample}:{h}'] = render_values(v)

            json_array.append(obj)

        if not any('ID' in obj for obj in json_array):
            # Remove ID column if none of the records actually had an ID.
            for obj in json_array:
                del obj['ID']

        normalized_json = normalize_json(self.info_fields, json_array)
        return json.dumps(normalized_json)


def normalize_json(info_fields, json_array):

    headers = ['unique_id', 'CHROM', 'POSITION', 'REF', 'ALT', 'ID']
    if info_fields is not None:
        for h in info_fields:
            if h == 'ANN':
                headers = headers + [ 'GENE', 'EFFECTS', 'IMPACT', 'TRANSCRIPT', 'GENE_ID', 'PROTEIN ALTERATION', 'DNA ALTERATION']
            else:
                headers.append(h)

    rows = []
    for json in json_array:
        r = []
        for h in headers:
            if h in json:
                r.append(json[h])
            else:
                r.append("")
        rows.append(r)

    return {
        "headers": headers,
        "rows": rows
    }




def render_value(v):
    """Render given value to string."""
    if v is None:
        return ""
    elif isinstance(v, float):
        # ensure that we don't waste space by insignificant digits
        return f'{v:.2g}'
    elif isinstance(v, str):
        str_val = v.replace('"', '')
        
        if str_val.startswith('http://') or str_val.startswith('https://'):
            return create_link(str_val)

    
    return html.escape(str(v))


def render_values(v):
    if v is None:
        return ""
    
    if isinstance(v, str) or isinstance(v, int) or isinstance(v, float):
        return render_value(v)
    return ','.join(map(render_value, v))


def render_id(v):
    if v.startswith('COSM'):
        return (f'<a href = "https://cancer.sanger.ac.uk/cosmic/mutation/overview?'
                f'id={v[4:]}" target="_blank">{v}</a>')
    return v


def render_ids(v):
    return ','.join(map(render_id, v.split(';')))


def decode_ann(variant):
    """Decode the standardized ANN field to something human readable."""
    annotations = ([variant.info['ANN'].split('|'
                   )] if isinstance(variant.info['ANN'],
                   str) else [e.split('|') for e in variant.info['ANN']])
    genes = []
    effects = []
    impacts = []
    transcripts = []
    gene_ids = []
    aa_alts = []
    nt_alts = []
    for allele in variant.alts:
        for ann in annotations:
            ann_allele, kind, impact, gene, gene_id = ann[:5]
            feature_id = ann[6]
            nt_mod, aa_mod = ann[9:11]

            if allele != ann_allele:
                continue

            full = '|'.join(ann)
            # Keep the most severe effect.
            # Link out to Genecards and show the full record in a tooltip.
            genes.append(gene)
            gene_ids.append(gene_id)
            effects.append(kind.replace('&', '/'))
            impacts.append(impact)
            transcripts.append(feature_id)
            aa_alts.append(aa_mod)
            nt_alts.append(nt_mod)
    return ','.join(genes), ','.join(effects), ','.join(impacts), ','.join(transcripts), ','.join(gene_ids), ','.join(aa_alts), ','.join(nt_alts)

def create_link(url):
    """Create an html link for the given url"""
    return (f'<a href = "{url}" target="_blank">{url}</a>')



import unittest
import pathlib
import io
from igv_reports.feature import FeatureReader, _NonIndexed, parse_gff, parse_bed

class FeatureFileTest(unittest.TestCase):

    def test_fetch(self):

        gff = str((pathlib.Path(__file__).parent / "data/minigenome/annotations.gtf.gz").resolve())
        feature_file = FeatureReader(gff)
        content = feature_file.slice()
        features = parse_gff(io.StringIO(content))
        self.assertTrue(len(features) == 2106)


    def test_query(self):

        gff = str((pathlib.Path(__file__).parent / "data/minigenome/annotations.gtf.gz").resolve())
        feature_file = FeatureReader(gff)
        content = feature_file.slice({"chr": "minigenome", "start": 5000, "end": 6000})
        features = parse_gff(io.StringIO(content))
        self.assertEqual(len(features), 33)

        # features should be sorted by start position, and overlap the query interval
        last_start = 0
        for f in features:
            self.assertTrue(f.end >= 5000 and f.start <= 6000)
            self.assertTrue(f.start >= last_start)
            last_start = f.start


    def test_nonindexed_fetch(self):

        gff = str((pathlib.Path(__file__).parent / "data/minigenome/annotations.gtf.gz").resolve())
        feature_file = _NonIndexed(gff)
        content = feature_file.slice()
        features = parse_gff(io.StringIO(content))
        self.assertTrue(len(features) == 2106)


    def test_nonindexed_query(self):

        gff = str((pathlib.Path(__file__).parent / "data/minigenome/annotations.gtf.gz").resolve())
        feature_file = _NonIndexed(gff)
        content = feature_file.slice({"chr": "minigenome", "start": 5000, "end": 6000})
        features = parse_gff(io.StringIO(content))
        self.assertEqual(len(features), 33)

        # features should be sorted by start position, and overlap the query interval
        last_start = 0
        for f in features:
            self.assertTrue(f.end >= 5000 and f.start <= 6000)
            self.assertTrue(f.start >= last_start)
            last_start = f.start



    def test_nongzipped_query(self):

        gff = str((pathlib.Path(__file__).parent / "data/minigenome/variants.bed").resolve())
        feature_file = FeatureReader(gff)
        content = feature_file.slice({"chr": "minigenome", "start": 4000, "end": 7000})
        features = parse_bed(io.StringIO(content))
        self.assertEqual(len(features), 3)





import pysam

def get_data(bam_file, region=None):
    args = ["-b", "-h",  bam_file]
    if region:
        range_string = region['chr'] + ":" + str(region['start']) + "-" + str(region['end'])
        args.append(range_string)
    return pysam.view(*args)


class BamReader:

    def __init__(self, filename):

        self.filename = filename


    def slice(self, region = None) :
        args = ["-b", "-h",  self.filename]
        if region:
            range_string = region['chr'] + ":" + str(region['start']) + "-" + str(region['end'])
            args.append(range_string)
        return pysam.view(*args)


import unittest

from igv_reports import tracks
from igv_reports import feature


class TrackTest(unittest.TestCase):

    def test_all(self):

        path = 'data/minigenome/variants.vcf.gz'
        expected = 'variants.vcf'
        name = tracks.get_name(path)
        self.assertEqual(name, expected)

        format = feature.infer_format(path)
        self.assertEqual('vcf', format)

        type = tracks.get_track_type(format)
        self.assertEqual('variant', type)






from . import regions
import pysam

def get_data(fasta_file,region=None):

    if None == region:

        with open(fasta_file,"r") as f:

            return(f.read())

    else :

        if isinstance(region,str):
            region = regions.parse_region(region)

        chr = region["chr"]
        start = region["start"] - 1
        end = region["end"]

        fasta = pysam.FastaFile(fasta_file)

        slice_seq = fasta.fetch(chr, start, end)

        fasta.close()

        return slice_seq




import unittest
import pathlib

from igv_reports import bam

class BAMTest(unittest.TestCase):


    def test_bam(self):


        region = {
            "chr": "minigenome",
            "start": 4000,
            "end": 10000
        }

        bam_file_path = str((pathlib.Path(__file__).parent / "data/minigenome/alignments.bam").resolve())
        data = bam.get_data(bam_file_path, region)
        self.assertTrue(data)


    def test_bam_noregion(self):

        bam_file_path = str((pathlib.Path(__file__).parent / "data/minigenome/alignments.bam").resolve())
        data = bam.get_data(bam_file_path)
        self.assertTrue(data)




def get_data(cyto_file, region):

    result = ''
    chr = region['chr'] if region else None
    with open(cyto_file) as f:

        for line in f:

            tokens = line.split('\t')

            if chr == None or tokens[0] == chr:

                result += line

    return result



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(name='igv-reports',
                 packages=['igv_reports'],
                 version='1.0.4',
                 description='Creates self-contained html pages for visual variant review with IGV (igv.js).',
                 long_description=long_description,
                 long_description_content_type="text/markdown",
                 license='MIT',
                 author='Jim Robinson',
                 url='https://github.com/igvteam/igv-reports',
                 keywords=['igv', 'bioinformatics', 'genomics', 'visualization', 'variant' ],
                 classifiers=[
                     'Programming Language :: Python :: 3',
                     'Development Status :: 4 - Beta ',
                     'Intended Audience :: Science/Research',
                     'Intended Audience :: Developers',
                     'License :: OSI Approved :: MIT License',
                     'Topic :: Scientific/Engineering :: Bio-Informatics '
                 ],
                 install_requires=[
                     'pysam', 'intervalTree', 'requests'
                 ],
                 entry_points={
                     'console_scripts': [
                         'create_report=igv_reports.report:main',
                         'create_datauri=igv_reports.datauri:main'
                     ],
                 },
                 package_data={'igv_reports': ['templates/*']},
                 )
